% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastcpd_wrappers.R
\name{fastcpd_binomial}
\alias{fastcpd_binomial}
\alias{fastcpd.binomial}
\title{Find change points efficiently in logistic regression models}
\usage{
fastcpd_binomial(data, ...)

fastcpd.binomial(data, ...)
}
\arguments{
\item{data}{A matrix or a data frame with the response variable as the first
column.}

\item{...}{Other arguments passed to \code{\link[=fastcpd]{fastcpd()}}, for example,
\code{segment_count}.}
}
\value{
A \linkS4class{fastcpd} object.
}
\description{
\code{\link[=fastcpd_binomial]{fastcpd_binomial()}} and \code{\link[=fastcpd.binomial]{fastcpd.binomial()}} are
wrapper functions of \code{\link[=fastcpd]{fastcpd()}} to find change points in
logistic regression models. The function is similar to \code{\link[=fastcpd]{fastcpd()}}
except that the data is by default a matrix or data frame with the response
variable as the first column and thus a formula is not required here.
}
\examples{
set.seed(1)
n <- 500
p <- 4
x <- mvtnorm::rmvnorm(n, rep(0, p), diag(p))
theta <- rbind(rnorm(p, 0, 1), rnorm(p, 2, 1))
y <- c(
  rbinom(300, 1, 1 / (1 + exp(-x[1:300, ] \%*\% theta[1, ]))),
  rbinom(200, 1, 1 / (1 + exp(-x[301:n, ] \%*\% theta[2, ])))
)
result <- suppressWarnings(fastcpd.binomial(cbind(y, x)))
summary(result)
plot(result)
}
\seealso{
\code{\link[=fastcpd]{fastcpd()}}
}
